/**
 * @class TheNotify
 * @param {Object} [options={}] - Optional settings
 */
'use strict'

const {get} = require('bwindow')

/** @lends TheNotify */
class TheNotify {
  constructor (options = {}) {
    const {
      scope,
    } = options
    this.workerScope = scope
  }

  async ask () {
    const Notification = get('Notification')
    if (!Notification) {
      console.warn(`Notification not supported`)
      return false
    }
    switch (Notification.permission) {
      case 'granted':
        return true
      case 'denied':
        return false
      default:
        const permission = await Notification.requestPermission()
        if (!Notification.permission) {
          Notification.permission = permission
        }
        return permission === 'granted'
    }
  }

  async send (message, options = {}) {
    const Notification = get('Notification')
    if (!Notification) {
      console.warn(`Notification not supported`)
      return null
    }
    try {
      const notification = new Notification(message, options)
      notification.onclick = options.onclick
      return notification
    } catch (e) {
      console.warn('[the-notify] Notification failed', e.message)
    }

    const hasServiceWorker = !!get('navigator.serviceWorker')
    if (hasServiceWorker) {
      await Notification.requestPermission()
      return this.sendByServiceWorker(message, options)
    }
    return null
  }

  async sendByServiceWorker (message, options = {}) {
    const {workerScope} = this
    const {tag} = options
    const serviceWorker = get('navigator.serviceWorker')
    const registration = await serviceWorker.getRegistration(workerScope)
    await registration.showNotification(message, options)
    return {
      set onclick (onclick) {
        registration.getNotifications({tag})
          .then((notifications) => {
            for (const notification of notifications) {
              notification.onclick = onclick
            }
          }
          )
      },
      close: async () => {
        const notifications = await registration.getNotifications({tag})
        for (const notification of notifications) {
          notification.close()
        }
      },
    }
  }
}

module.exports = TheNotify
